import bpy

# Global dictionary to store original sizes of background images
original_bg_image_scales = {}   

class OverscanPanel(bpy.types.Panel):
    bl_label = "Overscan"
    bl_idname = "RENDER_PT_overscan"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = 'output'

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        overscan = scene.overscan_settings

        layout.prop(overscan, "mode")
        if overscan.mode == 'PERCENTAGE':
            layout.prop(overscan, "percentage")
        elif overscan.mode == 'PIXELS':
            layout.prop(overscan, "extra_pixels")
        else:
            layout.prop(overscan, "specific_x_resolution")

        row = layout.row(align=True)
        row.operator("render.apply_overscan", icon='CHECKMARK')
        row.operator("render.revert_overscan", icon='LOOP_BACK')

class OverscanSettings(bpy.types.PropertyGroup):
    mode_items = [
        ('PERCENTAGE', "Percentage", ""),
        ('PIXELS', "Extra Pixels", ""),
        ('SPECIFIC_X', "Specific X Resolution", "")
    ]
    mode: bpy.props.EnumProperty(
        name="Mode",
        items=mode_items,
        description="Choose the method for overscan",
        default='PERCENTAGE'
    )
    percentage: bpy.props.FloatProperty(
        name = "Percentage",
        description = "Percentage of overscan to apply to the render",
        default = 0.0,
        min = 0.0,
        max = 100.0,
        subtype = 'PERCENTAGE'
    )
    extra_pixels: bpy.props.IntProperty(
        name="Extra Pixels",
        description="Number of extra pixels to add to the render dimensions",
        default=0,
        min=0
    )
    specific_x_resolution: bpy.props.IntProperty(
        name="Specific X Resolution",
        description="Set a specific X resolution while maintaining aspect ratio",
        default=1920,
        min=1
    )
    original_width: bpy.props.IntProperty(name="Original Width")
    original_height: bpy.props.IntProperty(name="Original Height")
    original_sensor_width: bpy.props.FloatProperty(name="Original Sensor Width")
    settings_stored: bpy.props.BoolProperty(name="Settings Stored", default=False)
     

class ApplyOverscan(bpy.types.Operator):
    bl_idname = "render.apply_overscan"
    bl_label = "Apply"

    def execute(self, context):
        scene = context.scene
        camera = scene.camera.data
        overscan = scene.overscan_settings

        # Check if overscan has already been applied
        if 'overscan_applied' in camera and camera['overscan_applied']:
            self.report({'WARNING'}, "Overscan already applied. Revert before applying again.")
            return {'CANCELLED'}

        # Store original values
        camera['original_width'] = scene.render.resolution_x
        camera['original_height'] = scene.render.resolution_y
        camera['original_sensor_width'] = camera.sensor_width

        # Apply overscan changes based on the selected mode
        if overscan.mode == 'PERCENTAGE':
            overscan_value = overscan.percentage / 100
            new_width = round(camera['original_width'] * (1 + overscan_value))
            new_height = round(camera['original_height'] * (1 + overscan_value))
        elif overscan.mode == 'PIXELS':
            new_width = camera['original_width'] + overscan.extra_pixels
            new_height = camera['original_height'] + overscan.extra_pixels
        else:  # SPECIFIC_X
            aspect_ratio = camera['original_height'] / camera['original_width']
            new_width = overscan.specific_x_resolution
            new_height = round(new_width * aspect_ratio)

        # Adjust render resolution
        scene.render.resolution_x = new_width
        scene.render.resolution_y = new_height

        # Adjust sensor width proportionally
        sensor_scale = new_width / camera['original_width']
        camera.sensor_width = camera['original_sensor_width'] * sensor_scale

        # Mark overscan as applied
        camera['overscan_applied'] = True

        return {'FINISHED'}

class RevertOverscan(bpy.types.Operator):
    bl_idname = "render.revert_overscan"
    bl_label = "Revert"

    def execute(self, context):
        scene = context.scene
        camera = scene.camera.data

        # Check if overscan has been applied
        if 'overscan_applied' not in camera or not camera['overscan_applied']:
            self.report({'WARNING'}, "No overscan to revert.")
            return {'CANCELLED'}

        # Revert to original values
        scene.render.resolution_x = camera['original_width']
        scene.render.resolution_y = camera['original_height']
        camera.sensor_width = camera['original_sensor_width']

        # Mark overscan as reverted
        camera['overscan_applied'] = False

        return {'FINISHED'}

def register():
    bpy.utils.register_class(OverscanPanel)
    bpy.utils.register_class(OverscanSettings)
    bpy.utils.register_class(ApplyOverscan)
    bpy.utils.register_class(RevertOverscan)
    bpy.types.Scene.overscan_settings = bpy.props.PointerProperty(type=OverscanSettings)

def unregister():
    bpy.utils.unregister_class(OverscanPanel)
    bpy.utils.unregister_class(OverscanSettings)
    bpy.utils.unregister_class(ApplyOverscan)
    bpy.utils.unregister_class(RevertOverscan)
    del bpy.types.Scene.overscan_settings

if __name__ == "__main__":
    register()
